#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#if defined(_WIN32)
#include <windows.h>
#endif

#include "version.h"
#include "lib_emui.h"
#include "global.h"


/* External variables */
/*---------------------------------------------------------------------*/
#if defined(__linux__)
int used_com   = -1;
#else
int used_com   = 0;
#endif

int is_monitor = 0;
const char *dio_port_str[4] = {"A", "B", "C", "D"};
const char *in_out_str  [2] = {"Out", "In"};

#if defined(__linux__)
/* in rs232.c */
#define RS232_PORTNR 52
extern char *comports[RS232_PORTNR];
#endif

#if defined(_WIN32)
EMUI_OPEN_DEVICE      		EMUIOpenDevice;
EMUI_SHOW_VER         		EMUIShowVer;
EMUI_RESET            		EMUIReset;
EMUI_CLOSE_DEVICE    	 	EMUICloseDevice;
EMUI_IO_CONFIG        		EMUIIOConfig;
EMUI_EXINTR_CONFIG    		EMUIExIntrConfig;
EMUI_EXP_CFG          		EMUIExpCfg;
EMUI_IMP_CFG          		EMUIImpCfg;
EMUI_CLEAR_EVENT      		EMUIClearEvent;
EMUI_MONITOR          		EMUIMonitor;
EMUI_READ_ALL_STATUS  		EMUIReadAllStatus;
EMUI_SET_AUTO_RET_ALL_STS  	EMUISetAutoRetAllSts;

/*---------------------------------------------------------------------*/
int load_emui_dll(void)
{
  HMODULE  hDLL = NULL;
  hDLL = LoadLibrary(LIB_EMUI);

  if(hDLL == NULL)
    return 1;
  else
  {
    EMUIOpenDevice    	  = (EMUI_OPEN_DEVICE)     		GetProcAddress(hDLL, "EMUIOpenDevice");
    EMUIShowVer       	  = (EMUI_SHOW_VER)        		GetProcAddress(hDLL, "EMUIShowVer");
    EMUIReset         	  = (EMUI_RESET)           		GetProcAddress(hDLL, "EMUIReset");
    EMUICloseDevice   	  = (EMUI_CLOSE_DEVICE)    		GetProcAddress(hDLL, "EMUICloseDevice");
    EMUIIOConfig      	  = (EMUI_IO_CONFIG)       		GetProcAddress(hDLL, "EMUIIOConfig");
    EMUIExIntrConfig  	  = (EMUI_EXINTR_CONFIG)   		GetProcAddress(hDLL, "EMUIExIntrConfig");
    EMUIExpCfg        	  = (EMUI_EXP_CFG)         		GetProcAddress(hDLL, "EMUIExpCfg");
    EMUIImpCfg        	  = (EMUI_IMP_CFG)         		GetProcAddress(hDLL, "EMUIImpCfg");
    EMUIClearEvent    	  = (EMUI_CLEAR_EVENT)     		GetProcAddress(hDLL, "EMUIClearEvent");
    EMUIMonitor       	  = (EMUI_MONITOR)         		GetProcAddress(hDLL, "EMUIMonitor");
    EMUIReadAllStatus 	  = (EMUI_READ_ALL_STATUS) 		GetProcAddress(hDLL, "EMUIReadAllStatus");
	EMUISetAutoRetAllSts  = (EMUI_SET_AUTO_RET_ALL_STS) GetProcAddress(hDLL, "EMUISetAutoRetAllSts");

    return 0;
  }

} /* END: load_emui_dll() */
#endif



/*---------------------------------------------------------------------*/
void print_menu (CMD_INFO cmd_info[], int used_com)
{
  int i;
  static bool is_first = true;

  /*------------------------------*/
  if(is_first)
  {
    printf("\n");
    printf(" EMUI DIO Manager %s\n", EMUI_CMD_UTILITY_VERSION);
    printf(" (C) 2005-2020 Innodisk Corp. Command Line Interface DIO Manager For EMUI.\n");
    printf(" -------------------------------------------------------------------------\n");
  }  
  is_first = false;


  /*------------------------------*/
  printf("\n");
  printf("Valid Commands:\n");
  printf("===============\n");


  /*------------------------------*/
  printf("(");
  #if defined(__linux__)
  if(used_com != -1)
    printf("Used: %s, ", comports[used_com]);
  #else
  if(used_com)
    printf("Used: COM%d, ", used_com);
  #endif
  else
    printf("Used: N/A, ");

  if(is_monitor)
    printf("Monitor: ON");
  else
    printf("Monitor: OFF");

  printf(")\n\n");
  /*------------------------------*/
  for(i=0; i<MGR_FUNC_NUM; i++)
    printf(" %s\n", cmd_info[i].cmd_name);
  printf("\n");

} /* END: print_menu() */



/*---------------------------------------------------------------------*/
unsigned char arr_2_byte(char arr[])
{  
  unsigned long ul_tmp;
  unsigned char rtn_byte;
  
  char str_tmp[5] = "0x";
  
  strcat(str_tmp, arr);
  ul_tmp = strtoul(str_tmp, NULL, 0);
  rtn_byte = (unsigned char) ul_tmp;

  return rtn_byte;

} /* END: arr_2_byte() */