#ifndef __LIB_EMUI_H__
#define __LIB_EMUI_H__

#include <stdbool.h>

#if defined(_WIN32)
#define    MAX_COM_NUM         256
#else
#define    MAX_COM_NUM         52
#endif

#define   DIO_PORT_NUM         4
#define   VER_LEN              16
#define   MAX_FILE_NAME_NUM    256

#define   _MAX_CMD_SEND_SIZE_  23
#define   _MAX_CMD_RTN_SIZE_   24
#define   _ENDBYTE1_           0x0D
#define   _ENDBYTE2_           0x0A




/* Monitor callback function */
typedef void (*MONITOR_CB_FUNC) (void *ptr);


/*-----------------------------------------------------*/
typedef struct
{
    /* IO config */
    int  dio_port;
    int  direction;
    unsigned char status;

    /* External interrupt */
    unsigned char ex_intr;

} GET_RTN;

/*-----------------------------------------------------*/
typedef struct
{
    int  com_port;
    char fw [VER_LEN];
    char api[VER_LEN];

} VER_INFO;

/*-----------------------------------------------------*/
typedef struct
{
    /* Set & Get config */
    int  com_port;
    int  dio_port;
    int  mode;

    /* Set config */
    int  dir_set;
    unsigned char sts_set;

    /* Get return info */
    GET_RTN  get_io_rtn; // O/P for user

} IO_INFO;

/*-----------------------------------------------------*/
typedef struct
{
    /* Set & Get config */
    int com_port;
    int mode;

    /* Set config */
    unsigned char exintr_set;    /*  bit0 ~ bit3: set edge, 0:=fall, 1:=rise
                                  *  bit4 ~ bit7: set enable or disable, 0:=disable, 1:=enable
                                  */
    /* Get return info */
    GET_RTN  get_exintr_rtn;

} EXINTR_INFO;

/*-----------------------------------------------------*/
typedef struct
{
    int mode;
    int com_port;

} EVENT_INFO;

/*-----------------------------------------------------*/
typedef struct
{
    int mode;
    int com_port;

    bool is_intr; // O/P
    bool is_eeerr; // O/P
    bool is_allsts; // O/P
    bool is_sts[DIO_PORT_NUM]; // O/P

    unsigned char rtn_sts_bfr[DIO_PORT_NUM]; // O/P
    unsigned char rtn_sts_now[DIO_PORT_NUM]; // O/P

    unsigned char rtn_intr; // O/P
    unsigned char rtn_eeerr; // O/P
    unsigned char rtn_all_sts[DIO_PORT_NUM]; // O/P

    MONITOR_CB_FUNC  monitor_cb_func; // I/P

    bool enable_intr; // I/P
    bool enable_ee; // I/P
    bool enable_allsts; // I/P
    unsigned char enable_sts_flag; // I/P
                                   /* flag 0x01: portA,
                                      flag 0x02: portB,
                                      flag 0x03: portA, portB
                                      ... */
} MONITOR_INFO;

/*-----------------------------------------------------*/
typedef struct
{
    int   com_port;
    char  file_name[MAX_FILE_NAME_NUM];

} CFG_FILE_INFO;


/*-----------------------------------------------------*/
/* DIO port select */
enum
{
    PORT_A = 1,
    PORT_B,
    PORT_C,
    PORT_D
}; 

/*-----------------------------------------------------*/
/* Mode select: Set/Get IO config */
enum
{
    SET_DIRECTION = 1,
    GET_DIRECTION,
    SET_STATUS,
    GET_STATUS,
    SET_STATUS_NR  /* No Record */
};

/*-----------------------------------------------------*/
/* In/Out select: Set/Get IO config */
enum
{
    SET_OUT = 0,
    SET_IN
};

/*-----------------------------------------------------*/
/* Mode select: External interrupt*/
enum 
{
    SET_EXT_INTR = 1,
    GET_EXT_INTR
};

/*-----------------------------------------------------*/
/* Mode select: Clear event */
enum 
{
    ALL_EVENT = 0,
    INTR_EVENT,
    EE_ERR_EVENT
};


/*-----------------------------------------------------*/
/* Mode select: Monitor */
enum 
{
    DISABLE = 0,
    ENABLE
}; 

#if defined(_WIN32)
typedef int  (*EMUI_OPEN_DEVICE)   			(int com_port);
typedef int  (*EMUI_SHOW_VER)      			(VER_INFO *ver_info);
typedef int  (*EMUI_RESET)          		(int com_port);
typedef int  (*EMUI_CLOSE_DEVICE)   		(int com_port);
typedef int  (*EMUI_IO_CONFIG)      		(IO_INFO *io_info);
typedef int  (*EMUI_EXINTR_CONFIG)  		(EXINTR_INFO *exintr_info);
typedef int  (*EMUI_EXP_CFG)        		(CFG_FILE_INFO *cfg_file_info);
typedef int  (*EMUI_IMP_CFG)        		(CFG_FILE_INFO *cfg_file_info);
typedef int  (*EMUI_CLEAR_EVENT)    		(EVENT_INFO *event_info);
typedef int  (*EMUI_MONITOR)        		(MONITOR_INFO *monitor_info);
typedef int  (*EMUI_BOOT_LOADER)    		(int com_port);
typedef int  (*EMUI_READ_ALL_STATUS)		(int com_port, unsigned char status[DIO_PORT_NUM]);
typedef int  (*EMUI_SET_AUTO_RET_ALL_STS)  	(int com_port, bool enable, int intervalTime);
#endif

#if defined(__linux__)
#ifdef __cplusplus 
extern "C" {
#endif 
int  EMUIOpenDevice   		(int com_port);
int  EMUIShowVer      		(VER_INFO *ver_info);
int  EMUIReset        		(int com_port);
int  EMUICloseDevice  		(int com_port);
int  EMUIIOConfig     		(IO_INFO *io_info);
int  EMUIExIntrConfig 		(EXINTR_INFO *exintr_info);
int  EMUIExpCfg       		(CFG_FILE_INFO *cfg_file_info);
int  EMUIImpCfg       		(CFG_FILE_INFO *cfg_file_info);
int  EMUIClearEvent   		(EVENT_INFO *event_info);
int  EMUIMonitor      		(MONITOR_INFO *monitor_info);
int  EMUIReadAllStatus		(int com_port, unsigned char status[DIO_PORT_NUM]);
int  EMUISetAutoRetAllSts  	(int com_port, bool enable, int intervalTime);
#ifdef __cplusplus 
}
#endif
#endif

#endif