Q               := @
CC_32           := gcc_32
CC_64           := gcc_64
SRCS            := $(wildcard *.c)
OBJS_32         := $(SRCS:.c=.o32)
OBJS_64         := $(SRCS:.c=.o64)
TARGET_32       := utility_emui_32.exe
TARGET_64       := utility_emui_64.exe
RC              := emui
RC_32           := utility_emui_32
RC_64           := utility_emui_64
CFLAGS          += -D_GNU_SOURCE -Wall -I./include
LDFLAGS_32      := $(LIBS_32)
LDFLAGS_64      := $(LIBS_64)
WINDRES_32      := windres_32
WINDRES_64      := windres_64

.PHONY: all clean

all: .depend ico_32 ico_64 $(TARGET_32) $(TARGET_64)

%.o32: %.c Makefile
	$(Q)echo "  Compiling '$<' ..."
	$(Q)$(CC_32) $(CFLAGS) -o $@ -c $<

%.o64: %.c Makefile
	$(Q)echo "  Compiling '$<' ..."
	$(Q)$(CC_64) $(CFLAGS) -o $@ -c $<

$(TARGET_32): $(OBJS_32)
	$(Q)echo -e "  $(COLOR_G)Building '$@' VER=$(AP_VER)... $(COLOR_W)"
	$(Q)$(CC_32) -o $@ $(OBJS_32) $(RC_32).res $(LDFLAGS_32)

$(TARGET_64): $(OBJS_64)
	$(Q)echo -e "  $(COLOR_G)Building '$@' VER=$(AP_VER)... $(COLOR_W)"
	$(Q)$(CC_64) -o $@ $(OBJS_64) $(RC_64).res $(LDFLAGS_64)

ico_32:
	$(Q)echo "  Generating '$(RC_32).res' ..."
	$(Q)$(WINDRES_32) $(RC).rc -O coff -o $(RC_32).res

ico_64:
	$(Q)echo "  Generating '$(RC_64).res' ..."
	$(Q)$(WINDRES_64) $(RC).rc -O coff -o $(RC_64).res

clean: 
	$(Q)echo "  Cleaning '$(TARGET_32)' & '$(TARGET_64)' ..."
	$(Q)rm -f .depend *~ $(OBJS_32) $(OBJS_64) $(TARGET_32) $(TARGET_64) *.bak *.res *.exe

.depend:
	$(Q)echo "  Generating '$@' ..."
	$(Q)$(CC_32) $(CFLAGS) -M *.c > $@
	$(Q)$(CC_64) $(CFLAGS) -M *.c > $@

ifeq (.depend, $(wildcard .depend))
	include .depend
endif

