#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

#if defined(__linux__)
#include <stdio_ext.h>
#include <unistd.h>
#else
#include <windows.h>
#endif

#include "version.h"
#include "lib_emui.h"
#include "global.h"


#if defined(__linux__)
#define MAX_COM_NUM        52
#define COM_PORT_NOT_OPEN  -1
#define COM_NOW_USE        used_com
extern char *comports[MAX_COM_NUM]; /* in rs232.c */
#else
/* windows max COM port number: 256 */
#define MAX_COM_NUM        256
#define COM_PORT_NOT_OPEN  0
#define COM_NOW_USE        used_com - 1
#endif


/************************ Function prototype ************************/
static int EMUIMgr (char *cmd_line);

static int EMUIMgrShowCOM       	  (char *argv[]);
static int EMUIMgrShowAll       	  (char *argv[]);
static int EMUIMgrOpenDevice    	  (char *argv[]);
static int EMUIMgrCloseDevice   	  (char *argv[]);
static int EMUIMgrShowVer       	  (char *argv[]);
static int EMUIMgrReset         	  (char *argv[]);
static int EMUIMgrIOConfig      	  (char *argv[]);
static int EMUIMgrExIntConfig   	  (char *argv[]);
static int EMUIMgrMonitor       	  (char *argv[]);
static int EMUIMgrClearEvent    	  (char *argv[]);
static int EMUIMgrExpCfg        	  (char *argv[]);
static int EMUIMgrImpCfg        	  (char *argv[]);
static int EMUIMgrTestPort      	  (char *argv[]);
static int EMUIMgrReadAllStatus 	  (char *argv[]);
static int EMUIMgrSetAutoRetAllSts  (char *argv[]);
static int EMUIMgrExitProg      	  (char *argv[]);

static void ev_cb_handler (void *ptr);

#if defined(__linux__)
static void delay_ms (unsigned int msec);
#endif

static bool com_not_open (void);
static void flush_stdin (void);

/************************ Global variables ************************/
static VER_INFO       ver_info;
static IO_INFO        io_info;
static EXINTR_INFO    exintr_info;
static EVENT_INFO     event_info;
static MONITOR_INFO   monitor_info;
static CFG_FILE_INFO  cfg_file_info;


/*-------------------------------*/
CMD_INFO cmd_info[MGR_FUNC_NUM] = 
{
  { EMUIMgrOpenDevice,    	 "OD [n] -------------------- Open COM port [n] DIO device"                      },
  { EMUIMgrCloseDevice,   	 "CD ------------------------ Close DIO device"                                  },
  { EMUIMgrShowCOM,       	 "SC ------------------------ Show available COM port"                           },
  { EMUIMgrShowAll,       	 "SA ------------------------ Show DIO all direction and status"                 },
  { EMUIMgrShowVer,       	 "SV ------------------------ Show DIO device FW & API version"                  },
  { EMUIMgrReset,         	 "RE ------------------------ Reset DIO device"                                  },
  { EMUIMgrIOConfig,      	 "IO ------------------------ Set/Get IO config of DIO device"                   },
  { EMUIMgrExIntConfig,   	 "EI ------------------------ Set/Get external interrupt of DIO device"          },
  { EMUIMgrMonitor,       	 "MO EN/DS ------------------ Enable/Disable monitor"                            },
  { EMUIMgrClearEvent,    	 "CR AL/IT/ER --------------- Clear all/intr/eeerr event"                        },
  { EMUIMgrExpCfg,        	 "EC \"filename\" ------------- Export DIO device config file with \"filename\"" },
  { EMUIMgrImpCfg,        	 "IC \"filename\" ------------- Import DIO device config file with \"filename\"" },
  { EMUIMgrTestPort,      	 "TP ------------------------ Test a port output status interval limitation"     },
  { EMUIMgrReadAllStatus, 	 "RA ------------------------ Return DIO all status"                             },
  { EMUIMgrSetAutoRetAllSts, "AR EN/DS ------------------ Enable/Disable auto return DIO all status"         },
  { EMUIMgrExitProg,      	 "EX ------------------------ Exit the program"                                  },
};


#if defined(_WIN32)
extern EMUI_OPEN_DEVICE     	  EMUIOpenDevice;
extern EMUI_SHOW_VER        	  EMUIShowVer;
extern EMUI_RESET           	  EMUIReset;
extern EMUI_CLOSE_DEVICE    	  EMUICloseDevice;
extern EMUI_IO_CONFIG       	  EMUIIOConfig;
extern EMUI_EXINTR_CONFIG   	  EMUIExIntrConfig;
extern EMUI_EXP_CFG         	  EMUIExpCfg;
extern EMUI_IMP_CFG         	  EMUIImpCfg;
extern EMUI_CLEAR_EVENT     	  EMUIClearEvent;
extern EMUI_MONITOR         	  EMUIMonitor;
extern EMUI_READ_ALL_STATUS 	  EMUIReadAllStatus;
extern EMUI_SET_AUTO_RET_ALL_STS  EMUISetAutoRetAllSts;
#endif


/* Main function */
/*===================================================================================================*/
int main (int argc, char *argv[])
{
  char  cmd_buf[CMD_BUF_NUM];
  int   status;
  int   len;

  #if defined(_WIN32)
  system("title EMUI DIO Manager"); /* set title */
  if(load_emui_dll()) /* load .dll */
  {
    printf("Load lib_emui.dll failed !\n");
    return 1;
  }
  #endif

  print_menu(cmd_info, used_com);
  printf(prompt_s);

  /* endless loop */
  while(fgets(cmd_buf, 256, stdin) != NULL)
  {
    len = strlen(cmd_buf);

    if(cmd_buf[len - 1] == '\n')
      cmd_buf[len - 1] = '\0';

    status = EMUIMgr(cmd_buf);
    memset(cmd_buf, 0, sizeof(cmd_buf));

    if(status == 100)
      break;
    else
      printf(prompt_s);

    flush_stdin();
  }

  return 0;

} /* END: main() */
/*===================================================================================================*/




/*---------------------------------------------------------------------------*/
static int EMUIMgr (char *cmd_buf)
{   
  int     i = 0, cnt = 0, status = 0;
  int     flag_i = 0;
  bool    flag_b = false;

  char    compare[2 + 1];
  char   *cmd_extract[CMD_BUF_NUM/2] = {NULL};
  char   *cmd_adjust;
  char   *cmd_strtok;
  char   *space = " ";

  char   *argv[MAX_CMD_NUM - 1] = {NULL};

  
  /* Adjustment 1: avoid "tab" or "space" for the begining */
  cmd_adjust = cmd_buf;
  while((*cmd_adjust == ' ') || (*cmd_adjust == '\t'))
    cmd_adjust++;


  /* Adjustment 2: extract separated command */
  for(i=0; i<strlen(cmd_adjust); i++)
  {
    if(*(cmd_adjust + i) == '\t')
      *(cmd_adjust + i) = ' ';
  }

  cmd_strtok     = strtok(cmd_adjust, space);
  cmd_extract[cnt] = cmd_strtok;

  while(cmd_strtok != NULL)
  {
    cnt++;
    cmd_strtok = strtok(NULL, space);
    cmd_extract[cnt] = cmd_strtok;
    if(cnt < 3)
      argv[cnt - 1] = cmd_extract[cnt]; 
  }

  /* Judgment & Execute command */
  if(cmd_extract[0] != NULL && cmd_extract[MAX_CMD_NUM] == NULL)
  {
    compare[0] = toupper(*(*(cmd_extract + 0) + 0));
    compare[1] = toupper(*(*(cmd_extract + 0) + 1));
    compare[2] = '\0';
    
    for(i=0; i<MGR_FUNC_NUM; i++)
    {
      if(strncmp(cmd_info[i].cmd_name, compare, 2) == 0 && (*(*(cmd_extract+0)+2)) == 0)
      {
        flag_b = true;
        flag_i = i;
      }
    }
    
    /* Valid cmd - callback function */
    if(flag_b)
    {
      status = (*(cmd_info[flag_i].cmd_call))(argv);
      return status;
    }

    /* Unknown cmd */
    else
    {
      print_menu(cmd_info, used_com);
      return status;
    }
  }

  /* Unknown command error */
  else
  {
    print_menu(cmd_info, used_com);
    return 0;
  }
  
  return status;

} /* END: EMUIMgr() */


/*---------------------------------------------------------------------------*/
static int EMUIMgrShowCOM (char *argv[])
{
  int  i;
  int  flag = -1;
  int  com_arr[MAX_COM_NUM];

  #if defined(__linux__)
  int  rtn = 1;

  /* Check valid cmd & extract COM port */
  if(argv[0] != NULL)
  {
    print_menu(cmd_info, used_com);
    return rtn;
  }
  #endif

  for(i=0; i<MAX_COM_NUM; i++)
  { 
    /* Porhibit open DIO port twice */
    #if defined(__linux__)
    if(i != used_com)
    #else
    if(i+1 != used_com)
    #endif
    {
      com_arr[i] = EMUIOpenDevice(i);
      EMUICloseDevice(i);
    }
    else
      flag = i;
  }

  #if defined(__linux__)
  printf("%-12s", prompt_r);
  #else
  printf(prompt_r);
  #endif
  printf("\n");

  for(i=0; i<MAX_COM_NUM; i++)
  {
    if(i == flag)
    {
      printf(prompt_a);
      #if defined(__linux__)
      printf("%s >> n=%d (Used now)\n", comports[i], i);
      #else
      printf("COM %-3d (Used now)\n", i+1);
      #endif
    }    
    else if(com_arr[i] == 0)
    {
      printf(prompt_a);
      #if defined(__linux__)
      printf("%s >> n=%d\n", comports[i], i);
      #else
      printf("COM %-3d (DIO)\n", i+1);
      #endif
    }
    #if defined(_WIN32)
    else if(com_arr[i] == 5)
    {
      printf(prompt_a);
      printf("COM %-3d (Used)\n", i+1);
    }
    else if(com_arr[i] == 6)
    {
      printf(prompt_a);
      printf("COM %-3d (Non DIO)\n", i+1);
    }
    #endif
  }

  return 0;

} /* END: EMUIMgrShowCOM() */


/*---------------------------------------------------------------------------*/
static int EMUIMgrShowAll (char *argv[])
{
  int i, com_port;
  int rtn = 1;

  bool failed_flag = false;

  int  dir[4];
  unsigned char sts[4];
  

  /* Check valid cmd & extract COM port */
  if(argv[0] != NULL)
  {
    print_menu(cmd_info, used_com);
    return rtn;
  }
  
  /* Check if COM port is open or not */
  if(com_not_open() == true)
  {
    return rtn;
  }
  else
  {
    com_port = COM_NOW_USE;
    io_info.com_port = com_port;
    io_info.mode     = GET_DIRECTION;
    
    for(i=PORT_A; i<PORT_D+1; i++)
    {
      io_info.dio_port = i;
      rtn = EMUIIOConfig(&io_info);
      
      /* Return command */
      if(rtn)
      {
        failed_flag = true;
        printf(prompt_r);
        printf("Get port %s IO failed !\n", dio_port_str[i-1]);
      }
      else
      {
        dir[i-1] = io_info.get_io_rtn.direction;
        sts[i-1] = io_info.get_io_rtn.status;
      }
    }
    
    if(!failed_flag)
    {
      printf(prompt_r);
      printf("Port  Dir   Sts\n");
      for(i=0; i<DIO_PORT_NUM; i++)
      {
        printf(prompt_a);
        printf("%-6s",  dio_port_str[i]);
        printf("%-6s", in_out_str[dir[i]]);
        printf("0x%02X\n", sts[i]);
      }
    }    
  }

  return rtn;


} /* END: EMUIMgrShowAll */


/*---------------------------------------------------------------------------*/
static int EMUIMgrOpenDevice (char *argv[])
{
  int com_port;
  int rtn = 1;

  #if defined(_WIN32)
  int chk_com = atoi(argv[0]);
  #endif

  #if defined(__linux__)
  if(argv[0] != NULL && (48 < argv[0][0] && argv[0][0] < 57))
    com_port = atoi(argv[0]);
  else
  {
    print_menu(cmd_info, used_com);
    return rtn;
  }
  #endif
  
  /* Check valid cmd & extract COM port */
  #if defined(__linux__)
  if(com_port < 0 || MAX_COM_NUM <= com_port || argv[1] != NULL)
  #else
  if(chk_com <= 0 || MAX_COM_NUM < chk_com || argv[1] != NULL)
  #endif
  {
    print_menu(cmd_info, used_com);
    return rtn;
  }

  #if defined(_WIN32)
  com_port = chk_com - 1;
  #endif

  /* Porhibit open DIO port twice */
  #if defined(__linux__)
  if(com_port != used_com)
  #else
  if(used_com != chk_com)
  #endif
    rtn = EMUIOpenDevice(com_port);
  else
  {
    printf(prompt_r);
    #if defined(__linux__)
    printf("%s has been open !\n", comports[com_port]);
    #else
    printf("COM %d port has been open !\n", chk_com);
    #endif
    return rtn;
  }

  /* Check return value */
  if(rtn == 0)
  {
    printf(prompt_r);
    #if defined(__linux__)
    printf("Open %s successfully !\n", comports[com_port]);
    used_com = com_port;
    #else
    printf("Open COM %d success !\n", chk_com);
    used_com = chk_com;
    #endif
  }
  else
  {
    #if defined(__linux__)
    printf("%s", prompt_r);
    printf("Open %s failed !\n", comports[com_port]);
    #else
    if(rtn == 5)
    {
      printf(prompt_r);
      printf("COM %d is in used !\n", chk_com);      
    }
    else if(rtn == 6)
    {
      printf(prompt_r);
      printf("COM %d is not EMUI DIO port !\n", chk_com);
      EMUICloseDevice(com_port);
    }
    else
    {
      printf(prompt_r);
      printf("Open COM %d failed !\n", chk_com);      
    }
    #endif
  }

  return rtn;

} /* END: EMUIMgrOpenDevice */


/*---------------------------------------------------------------------------*/
static int EMUIMgrCloseDevice(char *argv[])
{
  int rtn = 1;
  int com_port;

  /* Check valid cmd & extract COM port */
  if(argv[0] != NULL)
  {
    print_menu(cmd_info, used_com);
    return rtn;
  }

  /* Check if COM is open or not */
  if(used_com == COM_PORT_NOT_OPEN)
  {
    printf(prompt_r);
    printf("No COM port is open !\n");
  }

  /* API function */
  else
  {
    com_port = COM_NOW_USE;
    EMUICloseDevice(com_port);
    printf(prompt_r);

    #if defined(__linux__)
    printf("Close %s successfully !\n", comports[com_port]);
    used_com = -1;
    #else
    printf("Close COM %d successfully !\n", used_com);
    used_com = 0;
    #endif

    is_monitor = 0;

    return 0;
  }

  return rtn;

} /* END: EMUIMgrCloseDevice() */


/*---------------------------------------------------------------------------*/
static int EMUIMgrShowVer(char *argv[])
{
  int rtn = 1;

  /* Check valid cmd & extract COM port */
  if(argv[0] != NULL)
  {
    print_menu(cmd_info, used_com);
    return 0;
  }
  
  /* Check if COM port is open or not */
  if(used_com == COM_PORT_NOT_OPEN)
  {
    printf(prompt_r);
    printf("Please open COM port !\n");
    return 0;
  }
  
  /* Call API function */
  else
  {
    ver_info.com_port = COM_NOW_USE;
    rtn = EMUIShowVer(&ver_info);
    
    /* Return command */
    if(rtn)
    {
      printf(prompt_r);
      printf("Show version failed !\n");
      return rtn;
    }
    
    printf(prompt_r);
    printf("%s, %s\n", ver_info.fw, ver_info.api);
  }

  return rtn;


} /* END: EMUIMgrShowVer() */


/*---------------------------------------------------------------------------*/
static int EMUIMgrReset(char *argv[])
{
  int com_port;
  int rtn = 1;

  /* Check valid cmd & extract COM port */
  if(argv[0] != NULL)
  {
    print_menu(cmd_info, used_com);
    return rtn;
  }  

  /* Check if COM port is open or not */
  if(com_not_open() == true)
  {
    return rtn;
  }
  else
  {
    com_port = COM_NOW_USE;
    rtn = EMUIReset(com_port);

    /* Return command */
    if(rtn)
    {
      printf(prompt_r);
      printf("Reset to default failed !\n");
      return rtn;
    }
    else
    {
      printf(prompt_r);
      printf("Reset to default successfully !\n");      
    }
  }

  return rtn;

} /* END: EMUIMgrReset() */



/*---------------------------------------------------------------------------*/
static int EMUIMgrIOConfig (char *argv[])
{
  int   com_port;
  int   rtn = 1;
  int   dio_port, mode, dir;

  char  port_buf,   k_port_buf  [CMD_BUF_NUM] = {0};
  char  mode_buf,   k_mode_buf  [CMD_BUF_NUM] = {0};
  char  select_buf, k_select_buf[CMD_BUF_NUM] = {0};
  char  dir_buf,    k_in_out_buf[CMD_BUF_NUM] = {0};
  char  sts_buf[BYTE_CHAR_NUM + 1], k_sts_buf[CMD_BUF_NUM] = {0};

  unsigned char sts_byte;

  /* Check valid cmd & extract COM port */
  if(argv[0] != NULL)
  {
    print_menu(cmd_info, used_com);
    return rtn;
  }

  /* Check if COM port is open or not */
  if(com_not_open() == true)
  {
    return rtn;
  }
  else
  {
    /* Port select */
    do
    {
      flush_stdin();

      memset(k_port_buf, 0, sizeof(k_port_buf));
      printf("%sPort[A/B/C/D]: ", prompt_a);
      scanf ("%s", k_port_buf);
      port_buf = toupper(k_port_buf[0]);
      
      switch(port_buf)
      {
        case 'A':
                  dio_port = PORT_A;
                  break;
        case 'B':
                  dio_port = PORT_B;
                  break;
        case 'C':
                  dio_port = PORT_C;
                  break;
        case 'D':      
                  dio_port = PORT_D;
                  break;
      }

    } while(k_port_buf[1] != 0 || port_buf < 65 || 68 < port_buf);
    
    /* Mode select */
    do
    {
      flush_stdin();

      memset(k_mode_buf, 0, sizeof(k_mode_buf));
      printf("%sMode[<S>et/ <G>et]: ", prompt_a);
      scanf ("%s", k_mode_buf);
      mode_buf = toupper(k_mode_buf[0]);

    } while(k_mode_buf[1] != 0 || (mode_buf != 'S' && mode_buf != 'G'));

    /* Direction & Status */
    do
    {
      flush_stdin();

      memset(k_select_buf, 0, sizeof(k_select_buf));
      printf("%sSelect[<D>irection/ <S>tatus/ Status<N>oRecord]: ", prompt_a);
      scanf ("%s", k_select_buf);
      select_buf = toupper(k_select_buf[0]);

      if(mode_buf == 'S' && select_buf == 'D')
        mode = SET_DIRECTION;
      else if(mode_buf == 'G' && select_buf == 'D')
        mode = GET_DIRECTION;
      else if(mode_buf == 'S' && select_buf == 'S')
        mode = SET_STATUS;
      else if(mode_buf == 'G' && select_buf == 'S')
        mode = GET_STATUS;
      else if(mode_buf == 'S' && select_buf == 'N')
        mode = SET_STATUS_NR;

    } while(k_select_buf[1] != 0 || (select_buf != 'D' && select_buf != 'S' && select_buf != 'N'));


    /* Direction part */
    if(mode_buf == 'S')
    {
      if(select_buf == 'D')
      {
        /* in & out */
        do
        {
          flush_stdin();

          memset(k_in_out_buf, 0, sizeof(k_in_out_buf));
          printf("%sDirection[<I>n/<O>ut]: ", prompt_a);
          scanf ("%s", k_in_out_buf);
          dir_buf = toupper(k_in_out_buf[0]);

          if(dir_buf == 'I')
            dir = SET_IN;
          else if(dir_buf == 'O')
            dir = SET_OUT;        

        } while(k_in_out_buf[1] != 0 || (dir_buf != 'I' && dir_buf != 'O'));
      }

      /* high & low */
      if(dir_buf == 'O' || select_buf == 'S' || select_buf == 'N')
      {
        do
        {
          flush_stdin();

          memset(k_sts_buf, 0, sizeof(k_sts_buf));
          printf("%sStatus[00 ~ FF]: ", prompt_a);
          scanf ("%s", k_sts_buf);
          sts_buf[0] = toupper(k_sts_buf[0]);
          sts_buf[1] = toupper(k_sts_buf[1]);
          sts_buf[2] = '\0';

        } while( (k_sts_buf[2] != 0) ||
                  !((ASCII_0 <= sts_buf[0] && sts_buf[0] <= ASCII_9) || (ASCII_A <= sts_buf[0] && sts_buf[0] <= ASCII_F)) ||
                  !((ASCII_0 <= sts_buf[1] && sts_buf[1] <= ASCII_9) || (ASCII_A <= sts_buf[1] && sts_buf[1] <= ASCII_F)) );
       
        /* char arr[2] to byte */
        sts_byte = arr_2_byte(sts_buf);
      }
    }

    /* Setting API function arguments */
    com_port = COM_NOW_USE;
    io_info.com_port = com_port;
    io_info.mode     = mode;
    io_info.dio_port = dio_port;

    if(mode == SET_DIRECTION)
    {
      io_info.dir_set  = dir;
      io_info.sts_set  = sts_byte;
    }
    else if(mode == SET_STATUS || mode == SET_STATUS_NR)
      io_info.sts_set  = sts_byte;

    
    /* Call API function */
    /*-------------------------*/
    rtn = EMUIIOConfig(&io_info);
    /*-------------------------*/

    /* Return command */
    if(rtn > 0)
    {
      printf(prompt_r);
      if(mode_buf == 'S')
      {
        if(rtn == 1)        
          printf("Set IO failed !\n");
        else if(rtn == 2)
          printf("Invalid status setting with INPUT direction !\n");
      }
      else if(mode_buf == 'G')
        printf("Get IO failed !\n");

      return rtn;
    }
    else
    {
      if(mode == GET_DIRECTION)
      {
        printf(prompt_r);
        printf("Port: %s, ",       dio_port_str[io_info.get_io_rtn.dio_port - 1]);
        printf("Direction: %s, ",  in_out_str  [io_info.get_io_rtn.direction]);
        printf("Status: 0x%02X\n", io_info.get_io_rtn.status);        
      }
      else if(mode == GET_STATUS)
      {
        printf(prompt_r);
        printf("Port: %s, ",       dio_port_str[io_info.get_io_rtn.dio_port - 1]);
        printf("Status: 0x%02X\n", io_info.get_io_rtn.status);
      }
      else if(mode == SET_DIRECTION || mode == SET_STATUS || mode == SET_STATUS_NR)
      {
        printf(prompt_r);
        printf("Set IO successfully !\n");    
      }      
    }
  }

  return rtn;

} /* END: EMUIMgrIOConfig() */



/*---------------------------------------------------------------------------*/
static int EMUIMgrExIntConfig (char *argv[])
{
  int   com_port;
  int   rtn = 1;
  int   mode;

  char  mode_buf, k_mode_buf[CMD_BUF_NUM] = {0};
  char  edrf_buf[BYTE_CHAR_NUM + 1], k_edrf_buf[CMD_BUF_NUM] = {0}; /* edrf: enable, disable, rise, fall*/
  
  unsigned char edrf_byte;

  /* Check valid cmd & extract COM port */
  if(argv[0] != NULL)
  {
    print_menu(cmd_info, used_com);
    return rtn;
  }

  /* Check if COM port is open or not */
  if(com_not_open() == true)
  {
    return rtn;
  }
  else
  {
    /* Set & Get */
    do
    {
      flush_stdin();

      memset(k_mode_buf, 0, sizeof(k_mode_buf));
      printf("%sMode[<S>et/ <G>et]: ", prompt_a);
      scanf ("%s", k_mode_buf);
      mode_buf = toupper(k_mode_buf[0]);

      if(mode_buf == 'S')
        mode = SET_EXT_INTR;
      else if(mode_buf == 'G')
        mode = GET_EXT_INTR;

    } while(k_mode_buf[1] != 0 || (mode_buf != 'S' && mode_buf != 'G'));


    /* EN/DIS & Rise/Fall */
    if(mode_buf == 'S')
    { 
      do
      {
        flush_stdin();

        memset(k_edrf_buf, 0, sizeof(k_edrf_buf));
        printf("%sEn/Dis[7:4] Rise/Fall[3:0]: ", prompt_a);
        scanf ("%s", k_edrf_buf);
        edrf_buf[0] = toupper(k_edrf_buf[0]);
        edrf_buf[1] = toupper(k_edrf_buf[1]);
        edrf_buf[2] = '\0';

      } while( (k_edrf_buf[2] != 0) ||
                !((ASCII_0 <= edrf_buf[0] && edrf_buf[0] <= ASCII_9) || (ASCII_A <= edrf_buf[0] && edrf_buf[0] <= ASCII_F)) ||
                !((ASCII_0 <= edrf_buf[1] && edrf_buf[1] <= ASCII_9) || (ASCII_A <= edrf_buf[1] && edrf_buf[1] <= ASCII_F)) );
     
      /* char arr[2] to byte */
      edrf_byte = arr_2_byte(edrf_buf);
    }
    
    /* Setting API function arguments */
    com_port = COM_NOW_USE;
    exintr_info.com_port = com_port;
    exintr_info.mode     = mode;

    if(mode == SET_EXT_INTR)
      exintr_info.exintr_set = edrf_byte;
    
    /* Call API function */
    /*---------------------------------*/
    rtn = EMUIExIntrConfig(&exintr_info);
    /*---------------------------------*/

    /* Return command */
    if(rtn)
    {
      printf(prompt_r);
      if(mode_buf == 'S')
        printf("Set EX INTR failed !\n");
      else if(mode_buf == 'G')
        printf("Get EX INTR failed !\n");

      return rtn;
    }

    if(mode == GET_EXT_INTR)
    { 
      printf(prompt_r);
      printf("External Interrupt: 0x%02X\n", exintr_info.get_exintr_rtn.ex_intr);
    }
    else if(mode == SET_EXT_INTR)
    {
      printf(prompt_r);
      printf("Set EX INTR successfully !\n");
    }
  }  

  return rtn;
  

} /* END: EMUIMgrExIntConfig() */


/*---------------------------------------------------------------------------*/
static int EMUIMgrMonitor (char *argv[])
{
  int   com_port;
  int   rtn = 1;  
  int   mode;
  char  extract_buf[3];
  char *cmp_buf[2] = {"EN", "DS"};

  /* Check valid cmd & extract COM port */
  if(argv[0] == NULL || argv[1] != NULL)
  {
    print_menu(cmd_info, used_com);
    return rtn;
  }

  extract_buf[0] = toupper(*(*(argv + 0) + 0));
  extract_buf[1] = toupper(*(*(argv + 0) + 1));
  extract_buf[2] = '\0';

  /* Extract mode */
  if(!strcmp(extract_buf, cmp_buf[0]))
    mode = ENABLE;
  else if(!strcmp(extract_buf, cmp_buf[1]))
    mode = DISABLE;
  else
  {
    print_menu(cmd_info, used_com);
    return rtn;
  }

  /* Check if COM port is open or not */
  if(com_not_open() == true)
  {
    return rtn;
  }
  else
  {
    /* Setting API function arguments */
    com_port = COM_NOW_USE;
    monitor_info.com_port = com_port;
    monitor_info.mode = mode;

    if(mode == ENABLE)
    {
      // char sts_buf[3];
      char select_buf[128];
      bool select_result;

      /* select interrupt */
      /*----------------------------------------------*/
      do
      {
        flush_stdin();
        memset(select_buf, 0, 128);

        printf("%sEnable Interrupt[y/n]: ", prompt_a);
        scanf("%s", select_buf);

        switch(toupper(select_buf[0]))
        {
          case 'Y':
                    select_result = true;
                    break;
          case 'N':
                    select_result = false;
                    break;
        }

      } while(strlen(select_buf) != 1 || (toupper(select_buf[0]) != 'Y' && toupper(select_buf[0]) != 'N'));

      monitor_info.enable_intr = select_result;


      /* select EE error */
      /*----------------------------------------------*/
      do
      {
        flush_stdin();
        memset(select_buf, 0, 128);
        printf("%sEnable EE[y/n]: ", prompt_a);
        scanf("%s", select_buf);

        switch(toupper(select_buf[0]))
        {
          case 'Y':
                    select_result = true;
                    break;
          case 'N':
                    select_result = false;
                    break;
        }

      } while(strlen(select_buf) != 1 || (toupper(select_buf[0]) != 'Y' && toupper(select_buf[0]) != 'N' ));

      monitor_info.enable_ee = select_result;

      /* select status changed port */
      /*----------------------------------------------*/
      do
      {
        flush_stdin();
        memset(select_buf, 0, 128);
#if 0
        /* use SA */
        printf("%sStatus Port[Hex]: ", prompt_a);
        scanf("%s", select_buf);
        sts_buf[0] = toupper(select_buf[0]);
        sts_buf[1] = toupper(select_buf[1]);
        sts_buf[2] = '\0';

      } while((select_buf[2] != 0) || !(ASCII_0 == sts_buf[0]) ||
              !((ASCII_0 <= sts_buf[1] && sts_buf[1] <= ASCII_9) || (ASCII_A <= sts_buf[1] && sts_buf[1] <= ASCII_F)) );

      monitor_info.enable_sts_flag = arr_2_byte(sts_buf);
#else
        /* use RA */
        printf("%sEnable input status changed[y/n]: ", prompt_a);
        scanf("%s", select_buf);
        switch(toupper(select_buf[0]))
        {
          case 'Y':
                    select_result = true;
                    break;
          case 'N':
                    select_result = false;
                    break;
        }

      } while(strlen(select_buf) != 1 || (toupper(select_buf[0]) != 'Y' && toupper(select_buf[0]) != 'N' ));

      monitor_info.enable_sts_flag = select_result;
#endif

      /* assign callback function */
      monitor_info.monitor_cb_func = ev_cb_handler;

    }

    /* Call API function */
    /*---------------------------------------*/
    rtn = EMUIMonitor(&monitor_info);
    /*---------------------------------------*/

    /* Return command */
    if(rtn)
    {
      printf(prompt_r);
      if(mode == ENABLE)
        printf("EMUI monitor enable failed !\n");
      else if(mode == DISABLE)
        printf("EMUI monitor disable failed !\n");
    }
    else
    {
      printf(prompt_r);
      if(mode == ENABLE)
      {
        is_monitor = 1;
        printf("EMUI monitor enable successfully !\n");
      }
      else if(mode == DISABLE)
      {
        is_monitor = 0;
        printf("EMUI monitor disable successfully !\n");      
      }
    }
  }

  return rtn;
    

} /* END: EMUIMgrMonitor() */


/*---------------------------------------------------------------------------*/
static int EMUIMgrClearEvent (char *argv[])
{
  int   com_port;
  int   mode;
  int   rtn = 1;  
  char  extract_buf[3];
  char *cmp_buf[3] = {"AL", "IT", "ER"};

  /* Check valid cmd & extract COM port */
  if(argv[0] == NULL || argv[1] != NULL)
  {
    print_menu(cmd_info, used_com);
    return rtn;
  }

  /* Extract mode */
  extract_buf[0] = toupper(*(*(argv + 0) + 0));
  extract_buf[1] = toupper(*(*(argv + 0) + 1));
  extract_buf[2] = '\0';

  if(!strcmp(extract_buf, cmp_buf[0]))
    mode = ALL_EVENT;    
  else if(!strcmp(extract_buf, cmp_buf[1]))
    mode = INTR_EVENT;
  else if(!strcmp(extract_buf, cmp_buf[2]))
    mode = EE_ERR_EVENT;
  else
  {
    print_menu(cmd_info, used_com);
    return rtn;
  }

  /* Check if COM port is open or not */
  if(com_not_open() == true)
  {
    return rtn;
  }
  else
  {
    /* Setting API function arguments */
    com_port = COM_NOW_USE;
    event_info.com_port = com_port;
    event_info.mode = mode;

    /* Call API function */
    /*------------------------------*/
    rtn = EMUIClearEvent(&event_info);
    /*------------------------------*/

    /* Return command */
    if(rtn)
    {
      printf(prompt_r);
      printf("Clear event failed !\n");
    }
    else
    {
      printf(prompt_r);
      printf("Clear event successfully !\n");
    }
  }

  return rtn;
    

} /* END: EMUIMgrClearEvent() */


/*---------------------------------------------------------------------------*/
static int EMUIMgrExpCfg (char *argv[])
{
  int    com_port;
  int    rtn = 1;
  char  *extract_buf = argv[0];

  /* Check valid cmd & extract COM port */
  if(argv[0] == NULL || argv[1] != NULL)
  {
    print_menu(cmd_info, used_com);
    return rtn;
  }

  /* Check if COM port is open or not */
  if(com_not_open() == true)
  {
    return rtn;
  }
  else
  {
    /* Setting API function arguments */
    com_port = COM_NOW_USE;
    cfg_file_info.com_port = com_port;
    memset(cfg_file_info.file_name, 0, sizeof(cfg_file_info.file_name));
    memcpy(cfg_file_info.file_name, extract_buf, strlen(extract_buf));

    /* Call API function */
    /*-----------------------------*/
    rtn = EMUIExpCfg(&cfg_file_info);
    /*-----------------------------*/

    /* Return command */
    if(rtn)
    {
      printf(prompt_r);
      printf("Export config file failed !\n");
    }
    else
    {
      printf(prompt_r);
      printf("Export config file successfully !\n");
    }
  }

  return rtn;

} /* END: EMUIMgrExpCfg() */



/*---------------------------------------------------------------------------*/
static int EMUIMgrImpCfg (char *argv[])
{
  int    com_port;
  int    rtn = 1;  
  char  *extract_buf = argv[0];

  /* Check valid cmd & extract COM port */
  if(argv[0] == NULL || argv[1] != NULL)
  {
    print_menu(cmd_info, used_com);
    return 0;
  }

  /* Check if COM port is open or not */
  if(com_not_open() == true)
  {
    return rtn;
  }
  else
  {
    /* Setting API function arguments */
    com_port = COM_NOW_USE;
    cfg_file_info.com_port = com_port;
    memset(cfg_file_info.file_name, 0, sizeof(cfg_file_info.file_name));
    memcpy(cfg_file_info.file_name, extract_buf, strlen(extract_buf));

    /* Call API function */
    /*-----------------------------*/
    rtn = EMUIImpCfg(&cfg_file_info);
    /*-----------------------------*/

    /* Return command */
    if(rtn)
    {
      printf(prompt_r);
      printf("Import config failed !\n");
    }
    else
    {
      printf(prompt_r);
      printf("Import config file successfully !\n");
    }
  }

  return rtn;


} /* END: EMUIMgrImpCfg() */




/*---------------------------------------------------------------------------*/
static int EMUIMgrTestPort (char *argv[])
{
  int   rtn = 1;
  int   com_port;
  int   shift = 0;
  int   dio_port, total_time;
  int   itvl;
  char  port_buf, k_port_buf[CMD_BUF_NUM] = {0};
  char  k_itvl_buf[CMD_BUF_NUM] = {0};
  char  k_total_buf[CMD_BUF_NUM] = {0};
  clock_t  c_start;
  clock_t  c_itvl;

  /* Check valid cmd & extract COM port */
  if(argv[0] != NULL)
  {
    print_menu(cmd_info, used_com);
    return rtn;
  }

  /* Check if COM port is open or not */
  if(com_not_open() == true)
  {
    return rtn;
  }
  else
  {
    /* Port select */
    do
    {
      flush_stdin();

      memset(k_port_buf, 0, sizeof(k_port_buf));
      printf("%sPort[A/B/C/D]: ", prompt_a);
      scanf("%s", k_port_buf);
      port_buf = toupper(k_port_buf[0]);
      
      switch(port_buf)
      {
        case 'A':
                  dio_port = PORT_A;
                  break;
        case 'B':
                  dio_port = PORT_B;
                  break;
        case 'C':
                  dio_port = PORT_C;
                  break;
        case 'D':      
                  dio_port = PORT_D;
                  break;
      }

    } while(k_port_buf[1] != 0 || port_buf < 65 || 68 < port_buf);

    /* interval */
    do
    {
      flush_stdin();

      itvl = 0;
      memset(k_itvl_buf, 0, sizeof(k_itvl_buf));
      printf("%sInterval[ms]: ", prompt_a);
      scanf("%s", k_itvl_buf);
      itvl = atoi(k_itvl_buf);

    } while(itvl == 0);

    /* total time: must > 0 */
    do
    {
      flush_stdin();

      total_time = 0;
      memset(k_total_buf, 0, sizeof(k_total_buf));
      printf("%sTotal time[sec]: ", prompt_a);
      scanf("%s", k_total_buf);
      total_time = atoi(k_total_buf);

    } while(total_time <= 0);


    /* ----- if everything ok, start testing ----- */
    com_port = COM_NOW_USE;
    io_info.com_port = com_port;
    io_info.mode     = SET_DIRECTION;
    io_info.dio_port = dio_port;
    io_info.dir_set  = SET_OUT;
    io_info.sts_set  = 0x00;

    #if defined(__linux__)
    c_itvl = (clock_t) (total_time * 1000000);
    #else
    c_itvl = (clock_t) (total_time * 1000);
    #endif

    /* set direction out first */
    EMUIIOConfig(&io_info);

    /* set status & start timer */
    io_info.mode = SET_STATUS_NR;
    c_start = clock();
    while(1)
    {
      io_info.sts_set = 0x01 << shift;
      EMUIIOConfig(&io_info);

      /* itvl < 0: no delay */
      if(itvl > 0)
      {
        #if defined(__linux__)
        delay_ms(itvl);
        #else
        Sleep(itvl);
        #endif
      }

      shift++;
      if(shift == 8)
      {
        shift = 0;
      }

      if(clock() > (c_start + c_itvl))
      {
        break;
      }
    }
  }

  return 0;

} /* END: EMUIMgrTestPort() */


/*
 * return = 0 ... success
 * return = 1 ... failed
 * return = 2 ... illegal intervalTime value
 * return = 10 ... com port is not open
 *
 */
/*---------------------------------------------------------------------------*/
static int EMUIMgrSetAutoRetAllSts (char *argv[])
{
  int com_port;
  int rtn = 1;

  char extract_buf[3];
  char *cmp_buf[2] = {"EN", "DS"};

  bool select_result;

  int intervalTime;
  
  /* Check if COM port is open or not */
  if(com_not_open() == true)
  {
    return rtn;
  }
  else
  {
    com_port = COM_NOW_USE;
    io_info.com_port = com_port;
  }

  /* Check valid cmd & extract COM port */
  if(argv[0] == NULL || argv[1] != NULL)
  {
    print_menu(cmd_info, used_com);
    return rtn;
  }

  extract_buf[0] = toupper(*(*(argv + 0) + 0));
  extract_buf[1] = toupper(*(*(argv + 0) + 1));
  extract_buf[2] = '\0';

  /* Extract mode */
  if(!strcmp(extract_buf, cmp_buf[0]))
  {
    select_result = true;
    do
    {
      printf("IntervalTime(ms): ");
      scanf("%d", &intervalTime);
    } while (intervalTime < 0 || intervalTime > 0xFFFF);
   
  } 
  else if(!strcmp(extract_buf, cmp_buf[1]))
  {
    select_result = false;
  }
  else
  {
    print_menu(cmd_info, used_com);
    return rtn;
  }

  monitor_info.enable_allsts = select_result;

  if(0 == EMUISetAutoRetAllSts(com_port, select_result, intervalTime))
  {
    printf("EMUI auto return all status successfully !\n");
  }
  else
  {
    printf("EMUI auto return all status failed!\n");
  }
  return rtn;
} 
/* END: EMUIMgrSetAutoRetAllSts */


/*---------------------------------------------------------------------------*/
/*
 * return = 0 ... success
 * return = 1 ... failed
 * return = 10 ... com port is not open
 *
 */
/*---------------------------------------------------------------------------*/
static int EMUIMgrReadAllStatus (char *argv[])
{
  int i;
  int com_port;
  int rtn = 1;
  unsigned char sts[4];

  /* Check if COM port is open or not */
  if(com_not_open() == true)
  {
    return rtn;
  }
  else
  {
    com_port = COM_NOW_USE;
    io_info.com_port = com_port;
  }

  /* Check valid cmd & extract COM port */
  if(argv[0] != NULL)
  {
    print_menu(cmd_info, used_com);
    return rtn;
  }

/* Return command */

  if(0 == EMUIReadAllStatus(com_port, sts))
  {
    printf("EMUI return all status successfully!\n");
    printf(prompt_r);
    printf("Port  Sts\n");
    for(i=0; i<DIO_PORT_NUM; i++)
    {
      printf(prompt_a);
      printf("%-6s",  dio_port_str[i]);
      printf("0x%02X\n", sts[i]);
    }
  }
  else
  {
    printf("EMUI return all status failed!\n");
  }
  return rtn;
} 
/* END: EMUIMgrReadAllStatus */



/*---------------------------------------------------------------------------*/
static int EMUIMgrExitProg (char *argv[])
{
  if(used_com != COM_PORT_NOT_OPEN)
    EMUICloseDevice(COM_NOW_USE);

  return 100;

} /* END: EMUIMgrExitProg() */



/*---------------------------------------------------------------------------*/
static void ev_cb_handler (void *ptr)
{
  int           i;
  EVENT_INFO    event_info;
  MONITOR_INFO *monitor_info = (MONITOR_INFO *) ptr;

  event_info.com_port = COM_NOW_USE;

  /* Ex intr */
  if(monitor_info->is_intr)
  {
    printf("\n");
    printf(prompt_m);
    printf("External Interrupt: 0x%02X\n", monitor_info->rtn_intr);
    printf(prompt_s);
    event_info.mode = INTR_EVENT;
    EMUIClearEvent(&event_info);
  }

  /* EE error */
  if(monitor_info->is_eeerr)
  {
    printf("\n");
    printf(prompt_m);
    printf("EEPROM Error: 0x%02X\n", monitor_info->rtn_eeerr);
    printf(prompt_s);
    event_info.mode = EE_ERR_EVENT;
    EMUIClearEvent(&event_info);
  }

  /* All status */
  if(monitor_info->is_allsts)
  {
      printf("All Status: ");
      for(int i = 0; i < DIO_PORT_NUM; i++)
      {
          printf("\tPort %c=%02X", i+65, monitor_info->rtn_all_sts[i]);
      }
      printf("\n");
  }

  /* Status changed */
  if(monitor_info->enable_sts_flag)
  {
    for(i=0; i<DIO_PORT_NUM; i++)
    {
      if(monitor_info->is_sts[i])
      {
        printf("\n");
        printf(prompt_m);
        printf("Input Status Changed: dio port-> %s, befor->0x%02X, now->0x%02X\n",
                dio_port_str[i], monitor_info->rtn_sts_bfr[i], monitor_info->rtn_sts_now[i]);
        printf(prompt_s);
		    monitor_info->is_sts[i] = false;
        break;
      }
    }
  }

} /* END: ev_cb_handler() */


#if defined(__linux__)
/*---------------------------------------------------------------------------*/
static void delay_ms (unsigned int msec)
{
  clock_t goal = msec * CLOCKS_PER_SEC / 1000 + clock();  /* convert msecs to clock count */
  while(goal > clock());
}
#endif

/*---------------------------------------------------------------------------*/
static bool com_not_open (void)
{
  if(used_com == COM_PORT_NOT_OPEN)
  {
    printf(prompt_r);
    printf("Please open COM port !\n");
    return true;
  }

  return false;
}


/*---------------------------------------------------------------------------*/
static void flush_stdin (void)
{
  #if defined(__linux__)
  __fpurge(stdin);
  #else
  fflush(stdin);
  #endif
}