#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "lib_emui.h"


#if defined(_WIN32)
    #include <windows.h>
    #include <process.h>
    #if defined(_M_X64) || defined(__amd64__)
    #define LIB_EMUI "lib_emui_64.dll"
    #else
    #define LIB_EMUI "lib_emui_32.dll"
    #endif

    EMUI_OPEN_DEVICE   			EMUIOpenDevice;
    EMUI_SHOW_VER      			EMUIShowVer;
    EMUI_RESET         			EMUIReset;
    EMUI_CLOSE_DEVICE  			EMUICloseDevice;
    EMUI_IO_CONFIG     			EMUIIOConfig;
    EMUI_EXINTR_CONFIG 			EMUIExIntrConfig;
    EMUI_EXP_CFG       			EMUIExpCfg;
    EMUI_IMP_CFG       			EMUIImpCfg;
    EMUI_CLEAR_EVENT   			EMUIClearEvent;
    EMUI_MONITOR       			EMUIMonitor;
    EMUI_SET_AUTO_RET_ALL_STS  	EMUISetAutoRetAllSts;
    EMUI_READ_ALL_STATUS       	EMUIReadAllStatus;

    HMODULE hDLL = NULL;
    void load_emui_dll (void);
#endif

#if defined(__linux__)
    #include <signal.h>
    #include <unistd.h>

    char *dev_name[MAX_COM_NUM]={"/dev/ttyDIO0",   "/dev/ttyDIO1",   "/dev/ttyDIO2",   "/dev/ttyDIO3",   "/dev/ttyDIO4",  "/dev/ttyDIO5",
                                 "/dev/ttyDIO6",   "/dev/ttyDIO7",   "/dev/ttyDIO8",   "/dev/ttyDIO9",   "/dev/ttyDIO10", "/dev/ttyDIO11",
                                 "/dev/ttyDIO12",  "/dev/ttyDIO13",  "/dev/ttyDIO14",  "/dev/ttyDIO15",
                                 "/dev/ttyUSB0",   "/dev/ttyUSB1",   "/dev/ttyUSB2",   "/dev/ttyUSB3",   "/dev/ttyUSB4",  "/dev/ttyUSB5",
                                 "/dev/ttyAMA0",   "/dev/ttyAMA1",
                                 "/dev/ttyACM0",   "/dev/ttyACM1",   "/dev/ttyACM2",   "/dev/ttyACM3",   "/dev/ttyACM4",  "/dev/ttyACM5",
                                 "/dev/ttyACM6",   "/dev/ttyACM7",   "/dev/ttyACM8",   "/dev/ttyACM9",   "/dev/ttyACM10", "/dev/ttyACM11",
                                 "/dev/ttyACM12",  "/dev/ttyACM13",  "/dev/ttyACM14",  "/dev/ttyACM15",
                                 "/dev/rfcomm0",   "/dev/rfcomm1",   "/dev/ircomm0",   "/dev/ircomm1",
                                 "/dev/cuau0",     "/dev/cuau1",     "/dev/cuau2",     "/dev/cuau3",
                                 "/dev/cuaU0",     "/dev/cuaU1",     "/dev/cuaU2",     "/dev/cuaU3"};
#endif


int com_port = -1;
bool end_flag = false;
#if defined(_WIN32)
static BOOL end_emui (DWORD type);
#else
static void end_emui (int type);
#endif


/*-----------------------------------------------------------------*/
void monitor_cb_handler (void *ptr)
{
    EVENT_INFO     event_info;
    MONITOR_INFO  *monitor_info = (MONITOR_INFO *) ptr;

    event_info.com_port = monitor_info->com_port;

    if(monitor_info->is_intr)
    {
        printf("External Interrupt: 0x%02X\n", monitor_info->rtn_intr);
        event_info.mode = INTR_EVENT; /* ALL_EVENT */
        EMUIClearEvent(&event_info);
    }

    if(monitor_info->is_eeerr)
    {
        printf("EEPROM Error: 0x%02X\n", monitor_info->rtn_eeerr);
        event_info.mode = EE_ERR_EVENT; /* ALL_EVENT */
        EMUIClearEvent(&event_info);
    }

    if(monitor_info->is_allsts)
    {
        printf("All Status: ");
        for(int i=0; i<DIO_PORT_NUM; i++)
            printf("\tPort %c=%02X", i+65, monitor_info->rtn_all_sts[i]);
        printf("\n");
    }

    if(monitor_info->enable_sts_flag)
    {
        for(int i=0; i<DIO_PORT_NUM; i++)
        {
            if(monitor_info->is_sts[i])
            {
                printf("Input Status Changed: dio port-> %c, befor->0x%02X, now->0x%02X\n",
                        i + 65, monitor_info->rtn_sts_bfr[i], monitor_info->rtn_sts_now[i]);
                monitor_info->is_sts[i] = false;               
            }
        }
    }
}



/*-----------------------------------------------------------------*/
int main (int argc, char *argv[])
{
    int            i;
    int            rtn;
	int            intervalTime = 1000; /* [ms] */
    int            dio_port[DIO_PORT_NUM] = {PORT_A, PORT_B, PORT_C, PORT_D};
    int            dir[DIO_PORT_NUM] = {SET_OUT, SET_IN, SET_OUT, SET_IN};
    int            sts[DIO_PORT_NUM] = {0x55, 0xAA, 0xFF, 0x00};
    char          *config_name = "IOcfg";
	unsigned char  status[DIO_PORT_NUM];
    VER_INFO       ver_info;
    IO_INFO        io_info;
    CFG_FILE_INFO  cfg_file_info;
    EXINTR_INFO    exintr_info;
    MONITOR_INFO   monitor_info;

    #if defined(_WIN32)
    load_emui_dll();
    #endif


    /* EMUIOpenDevice() */
    /*----------------------------------------------------*/
    for(i=0; i<MAX_COM_NUM; i++)
    {
        rtn = EMUIOpenDevice(i);
        if(rtn == 0)
        {
            com_port = i;
            break;
        }
    }

    if(rtn)
    {
        printf("No EMUI-0D01 device!\n");
        return 1;
    }
    else
    {
        #if defined(_WIN32)
        printf("Open COM%d successfully!\n", com_port+1);
        #else
        printf("Open %s successfully!\n", dev_name[com_port]);
        #endif
    }

    #if defined(_WIN32)
    SetConsoleCtrlHandler((PHANDLER_ROUTINE) end_emui, TRUE);
    #else
    signal(SIGINT, end_emui);
    #endif

    /* EMUIShowVer() */
    /*----------------------------------------------------*/
    ver_info.com_port = com_port;
    rtn = EMUIShowVer(&ver_info);

    if(rtn)
    {
        printf("EMUIShowVer() failed!\n");
    }
    else
    {
        printf("EMUIShowVer() successfully!\n");
        printf("%s\n%s\n", ver_info.fw, ver_info.api);
    }

    /* EMUIIOConfig() */
    /*----------------------------------------------------*/
    rtn = 0;
    io_info.com_port = com_port;
    io_info.mode = SET_DIRECTION;
    for(i=0; i<DIO_PORT_NUM; i++)
    {
        io_info.dio_port = dio_port[i];
        io_info.dir_set = dir[i];
        io_info.sts_set = sts[i];

        rtn += EMUIIOConfig(&io_info);
    }

    if(rtn)
    {
        printf("Set EMUIIOConfig() failed!\n");
    }
    else
    {
        printf("Set EMUIIOConfig() successfully!\n");
    }

    /* get */
    io_info.mode = GET_DIRECTION;
    for(i=0; i<DIO_PORT_NUM; i++)
    {
        io_info.dio_port = dio_port[i];
        rtn = EMUIIOConfig(&io_info);
        if(rtn)
        {
            printf("Get EMUIIOConfig() failed!\n");
        }
        else
        {
            printf("--------------------------------\n");
            printf("Get EMUIIOConfig() successfully!\n");
            printf("Port: %d\n", io_info.get_io_rtn.dio_port);
            printf("Direction: %d\n", io_info.get_io_rtn.direction);
            printf("Status: %02X\n", io_info.get_io_rtn.status);
        }
    }


    /* EMUIExIntConfig() */
    /*----------------------------------------------------*/
    /* set */
    exintr_info.com_port = com_port;
    exintr_info.mode = SET_EXT_INTR;
    exintr_info.exintr_set = 0xFF; /* [7:4]: enable=1, disable=0, [3:0]: rise=1, fall=0 */
    rtn = EMUIExIntrConfig(&exintr_info);

    if(rtn)
    {
        printf("Set EMUIExIntrConfig() failed!\n");
    }
    else
    {
        printf("Set EMUIExIntrConfig() successfully!\n");
    }

    /* get */
    exintr_info.com_port = com_port;
    exintr_info.mode = GET_EXT_INTR;
    rtn = EMUIExIntrConfig(&exintr_info);

    if(rtn)
    {
        printf("Get EMUIExIntrConfig() failed!\n");
    }
    else
    {
        printf("Get EMUIExIntrConfig() successfully!\n");
        printf("EX INTR: 0x%02X\n", exintr_info.get_exintr_rtn.ex_intr);
    }

    /* EMUIExpCfg() */
    /*----------------------------------------------------*/
    cfg_file_info.com_port = com_port;
    strcpy(cfg_file_info.file_name, config_name);
    rtn = EMUIExpCfg(&cfg_file_info);

    if(rtn)
    {
        printf("EMUIExpCfg() failed!\n");
    }
    else
    {
        printf("EMUIExpCfg() successfully!\n");
    }


    /* EMUIReset() */
    /*----------------------------------------------------*/
    rtn = EMUIReset(com_port);

    if(rtn)
    {
        printf("EMUIReset() failed!\n");
    }
    else
    {
        printf("EMUIReset() successfully!\n");
    }

    /* EMUIImpCfg() */
    /*----------------------------------------------------*/
    cfg_file_info.com_port = com_port;
    strcpy(cfg_file_info.file_name, config_name);
    rtn = EMUIImpCfg(&cfg_file_info);

    if(rtn)
    {
        printf("EMUIImpCfg() failed!\n");
    }
    else
    {
        printf("EMUIImpCfg() successfully!\n");
    }

    /* EMUISetAutoRetAllSts() */
    /*----------------------------------------------------*/
    rtn = EMUISetAutoRetAllSts(com_port, true, intervalTime);
    
    if(rtn)
    {
        printf("EMUISetAutoRetAllSts() failed!\n");
    }
    else
    {
        printf("EMUISetAutoRetAllSts() successfully!\n");
    }

    /* EMUIReadAllStatus() */
    /*----------------------------------------------------*/
    rtn = EMUIReadAllStatus(com_port, status);
    
    if(rtn)
    {
        printf("EMUIReadAllStatus() failed!\n");
    }
    else
    {
        printf("EMUIReadAllStatus() successfully!\n");
        printf("All Status: ");
        for(int i=0; i<DIO_PORT_NUM; i++)
            printf("\tPort %c=%02X", i+65, status[i]);
        printf("\n");
    }

    /* EMUIMonitor() */
    /*----------------------------------------------------*/
    memset(&monitor_info, 0, sizeof(monitor_info));
    monitor_info.com_port = com_port;
    monitor_info.mode = ENABLE;
    monitor_info.monitor_cb_func = monitor_cb_handler;
    monitor_info.enable_intr = true;
    monitor_info.enable_ee = true;
    monitor_info.enable_allsts = true;
    monitor_info.enable_sts_flag = 0x0F; /* flag 0x00: do NOT monitor status changed
                                            flag 0x01: monitor portA
                                            flag 0x02: monitor portB
                                            flag (0x01 | 0x02): monitor portA & portB
                                            ... */

    rtn = EMUIMonitor(&monitor_info);
    if(rtn)
    {
        printf("EMUIMonitor() failed!\n");
    }
    else
    {
        printf("EMUIMonitor() successfully!\n");
    }

    while(end_flag == false)
    {
        #if 0
        static unsigned char counter = 0xFF;

        io_info.com_port = com_port;
        io_info.mode = SET_STATUS_NR;
        io_info.dio_port = PORT_A;
        io_info.sts_set  = counter;

        EMUIIOConfig(&io_info);
        printf("counter value 0x%02X\n", counter);

        counter--;
        if(counter == 0)
        {
            counter = 0xFF;
        }
        #endif

        #if defined(_WIN32)
        Sleep(1000);
        #else
        sleep(1);
        #endif
    }

    /* EMUICloseDevice() */
    /*----------------------------------------------------*/
    EMUICloseDevice(com_port);

    #if defined(_WIN32)
    FreeLibrary(hDLL);
    #endif
    return 0;
}




/*-----------------------------------------------------------------*/
#if defined(_WIN32)
static BOOL end_emui (DWORD type)
#else
static void end_emui (int type)
#endif
{
    #if defined(_WIN32)
    if(type == CTRL_C_EVENT)
    #else
    if(type == SIGINT)
    #endif
    {
        end_flag = true;
        printf("\nExit the program ...\n");
    }

    #if defined(_WIN32)
    return TRUE;
    #endif
}


#if defined(_WIN32)
/*-----------------------------------------------------------------*/
void load_emui_dll (void)
{
    hDLL = LoadLibrary(LIB_EMUI);

    if(hDLL == NULL)
    {
        printf("Load %s failed!\n", LIB_EMUI);
        exit(1);
    }
    else
    {
        EMUIOpenDevice   = (EMUI_OPEN_DEVICE)   GetProcAddress(hDLL, "EMUIOpenDevice");
        EMUIShowVer      = (EMUI_SHOW_VER)      GetProcAddress(hDLL, "EMUIShowVer");
        EMUIReset        = (EMUI_RESET)         GetProcAddress(hDLL, "EMUIReset");
        EMUICloseDevice  = (EMUI_CLOSE_DEVICE)  GetProcAddress(hDLL, "EMUICloseDevice");
        EMUIIOConfig     = (EMUI_IO_CONFIG)     GetProcAddress(hDLL, "EMUIIOConfig");
        EMUIExIntrConfig = (EMUI_EXINTR_CONFIG) GetProcAddress(hDLL, "EMUIExIntrConfig");
        EMUIExpCfg       = (EMUI_EXP_CFG)       GetProcAddress(hDLL, "EMUIExpCfg");
        EMUIImpCfg       = (EMUI_IMP_CFG)       GetProcAddress(hDLL, "EMUIImpCfg");
        EMUIClearEvent   = (EMUI_CLEAR_EVENT)   GetProcAddress(hDLL, "EMUIClearEvent");
        EMUIMonitor      = (EMUI_MONITOR)       GetProcAddress(hDLL, "EMUIMonitor");
        EMUISetAutoRetAllSts = (EMUI_SET_AUTO_RET_ALL_STS) GetProcAddress(hDLL, "EMUISetAutoRetAllSts");
        EMUIReadAllStatus = (EMUI_READ_ALL_STATUS) GetProcAddress(hDLL, "EMUIReadAllStatus");

        if(!EMUIOpenDevice   || !EMUIShowVer || !EMUIReset  || !EMUICloseDevice || !EMUIIOConfig ||
           !EMUIExIntrConfig || !EMUIExpCfg  || !EMUIImpCfg || !EMUIClearEvent  || !EMUIMonitor  ||
           !EMUISetAutoRetAllSts || !EMUIReadAllStatus)
        {
            printf("Get function from %s failed!\n", LIB_EMUI);
            exit(1);
        }
    }
}
#endif